import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// Create a [StatefulWidget] using [Provider]. Once created, the widget tree never change.
class StatefulImmutablePVC<T> extends StatefulWidget {
  /// The controller for this view.
  final T controller;

  /// Build a widget tree based on the [controller] value. Once created, the widget
  /// never rebuild itself.
  ///
  /// Must not be null.
  final Widget Function(BuildContext context, T controller, Widget child)
      builder;

  /// Called when this object is inserted into the tree.
  ///
  /// The framework will call this method exactly once for each [StatefulImmutablePVC] object
  /// it creates.
  final Function(T) onModelReady;

  /// Create a [StatefulWidget] using [Provider], and bind the [controller] to it.
  ///
  /// `StatefulImmutablePVC` is immutable, as the name suggests. That means it will never
  /// rebuild itself even if the controller changes its state.
  ///
  /// If you want the widget tree to rebuild when the [controller] changes its state,
  /// use [StatelessMutablePVC] or [StatefulMutablePVC] instead.
  ///
  /// [onModelReady] is called when this object is inserted into the tree. The framework
  /// will call this method exactly once for each [StatefulImmutablePVC] object it creates.
  StatefulImmutablePVC({
    @required this.controller,
    @required this.builder,
    @required this.onModelReady,
  });

  @override
  _StatefulImmutablePVCState createState() => _StatefulImmutablePVCState<T>();
}

class _StatefulImmutablePVCState<T> extends State<StatefulImmutablePVC<T>> {
  @override
  void initState() {
    if (widget.onModelReady != null) {
      widget.onModelReady(widget.controller);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Provider<T>(
      builder: (context) => widget.controller,
      child: Consumer<T>(builder: widget.builder),
      // dispose: (_, model) =>,
    );
  }
}
