import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// Create a [StatelessWidget] using [Provider]. Once created, the widget tree never change.
class StatelessImmutablePVC<T> extends StatelessWidget {
  /// The controller for this view.
  final T controller;

  /// Build a widget tree based on the [controller] value. Once created, the widget
  /// never rebuild itself.
  ///
  /// Must not be null.
  final Widget Function(BuildContext context, T controller, Widget child)
      builder;

  /// Create a [StatelessWidget] using [Provider], and bind the [controller] to it.
  ///
  /// `StatelessImmutablePVC` is immutable, as the name suggests. That means it will never
  /// rebuild itself even if the controller changes its state.
  ///
  /// If you want the widget tree to rebuild when the [controller] changes its state,
  /// use [StatelessMutablePVC] or [StatefulMutablePVC] instead.
  ///
  /// If you want to perform some initialization on the [controller] when the widget is
  /// inserted into the tree, use [StatefulImmutablePVC] instead.
  StatelessImmutablePVC({
    @required this.controller,
    @required this.builder,
  });

  @override
  Widget build(BuildContext context) {
    return Provider<T>(
      builder: (context) => controller,
      child: Consumer<T>(builder: builder),
      // dispose: (_, model) =>,
    );
  }
}
